@@ -1,26 +1,9 @@
 
-"""Class Quantity - Implements values at each triangular element
 
-To create:
-
-   Quantity(domain, vertex_values)
-
-   domain: Associated domain structure. Required.
-
-   vertex_values: N x 3 array of values at each vertex for each element.
-                  Default None
-
-   If vertex_values are None Create array of zeros compatible with domain.
-   Otherwise check that it is compatible with dimensions of domain.
-   Otherwise raise an exception
-
-   For Quantities that need to be saved during checkpointing, set register=True. Registered
-   Quantities can be found in the dictionary domain.quantities (note, other Quantities can
-   exist).
-"""
 
 import types
 import os.path
+from anuga.coordinate_transforms.geo_reference import Geo_reference
 
 from anuga.utilities.numerical_tools import ensure_numeric, is_scalar
 from anuga.geometry.polygon import inside_polygon
@@ -36,11 +19,39 @@ import numpy as num
 
 
 class Quantity(object):
+    """Class Quantity - Implements values at each triangular element
+    """
 
 
     counter = 0
 
     def __init__(self, domain, vertex_values=None, name=None, register=False):
+        """Create Quantity object
+        
+        :param domain: Associated domain structure. Required.
+        :param vertex_values: N x 3 array of values at each vertex for each element. Default None
+        :param str name: Provides a way to refer to a created quantity
+        :param register: Register a quantity
+
+
+        Usage:
+
+        >>> Quantity(domain, name="newQ", register=True)
+
+
+        If vertex_values are None Create array of zeros compatible with domain.
+        Otherwise check that it is compatible with dimensions of domain.
+        Otherwise raise an exception
+
+        For Quantities that need to be saved during checkpointing, set register=True. Registered
+        Quantities can be found in the dictionary domain.quantities (note, other Quantities can
+        exist).
+        
+        
+        """
+
+
+
         from anuga.abstract_2d_finite_volumes.generic_domain \
                             import Generic_Domain
 
@@ -1341,10 +1352,16 @@ class Quantity(object):
             x,y,Z = dem2array(filename)
 
         if location == 'centroids':
-            points = self.domain.centroid_coordinates
-
+            if indices is None:
+                points = self.domain.centroid_coordinates
+            else:
+                points = self.domain.centroid_coordinates[indices]
         else:
-            points = self.domain.vertex_coordinates
+            if indices is None:
+                points = self.domain.vertex_coordinates
+            else:
+                indices = num.array(indices)
+                points = self.domain.vertex_coordinates[tuple(indices),:]
 
         from anuga.geospatial_data.geospatial_data import Geospatial_data,  ensure_absolute
 
@@ -1384,30 +1401,35 @@ class Quantity(object):
                 self.vertex_values[:] = values.reshape((-1,3))
             else:
                 msg = 'Number of values must match number of indices'
-                assert values.shape[0] == indices.shape[0], msg
+                assert values.shape[0] == len(indices), msg
 
                 # Brute force
-                self.vertex_values[indices] = values.reshape((-1,3))
+                self.vertex_values.reshape(-1,)[indices] = values
             # Cleanup centroid values
             self.interpolate()
 
 
     def set_values_from_utm_raster(self,
                              raster,
-                             location='vertices',
+                             location='centroids',
                              indices=None,
                              verbose=False):
 
 
-
         x,y,Z = raster
 
-
         if location == 'centroids':
-            points = self.domain.centroid_coordinates
-
+            if indices is None:
+                points = self.domain.centroid_coordinates
+            else:
+                points = self.domain.centroid_coordinates[indices]
         else:
-            points = self.domain.vertex_coordinates
+            if indices is None:
+                points = self.domain.vertex_coordinates
+            else:
+                indices = num.array(indices)
+                points = self.domain.vertex_coordinates[tuple(indices),:]
+
 
         from anuga.geospatial_data.geospatial_data import ensure_absolute
 
@@ -1447,11 +1469,10 @@ class Quantity(object):
                 self.vertex_values[:] = values.reshape((-1,3))
             else:
                 msg = 'Number of values must match number of indices'
-                assert values.shape[0] == indices.shape[0], msg
+                assert values.shape[0] == len(indices), msg
 
                 # Brute force
-                self.vertex_values[indices] = values.reshape((-1,3))
-
+                self.vertex_values.reshape(-1,)[indices] = values
             # Cleanup centroid values
             self.interpolate()
 
@@ -1459,42 +1480,35 @@ class Quantity(object):
 
     def set_values_from_lat_long_grid_file(self,
                              filename,
-                             location='vertices',
+                             location='centroids',
                              indices=None,
+                             northern=False,
                              verbose=False):
 
-        """Read Digital Elevation model from the following ASCII format (.asc or .grd)
+        """Read Digital model from the following ASCII format (.asc or .grd)
 
         Example:
-        ncols         3121
-        nrows         1800
-        xllcorner     722000
-        yllcorner     5893000
-        cellsize      25
+        ncols         41
+        nrows         41
+        xllcorner     140
+        yllcorner     -30
+        cellsize      0.025
         NODATA_value  -9999
-        138.3698 137.4194 136.5062 135.5558 ..........
+        28.6 28.6 28.6 28.6 28.7 28.7 28.7 28.7 28.7 28.7 ....
 
+        This file would represent raster data from lower left corner 
+        at 140 long and -30 lat over to upper right corner 141 long -29 lat. 
 
-        An accompanying file with same basename but extension .prj must exist
-        and is used to fix the UTM zone, datum, false northings and eastings.
+        Here cellsize = 0.025 represents 1/40 of a degree
 
-        The prj format is assumed to be as
-
-        Projection    UTM
-        Zone          56
-        Datum         WGS84
-        Zunits        NO
-        Units         METERS
-        Spheroid      WGS84
-        Xshift        0.0000000000
-        Yshift        10000000.0000000000
-        Parameters
+        :param str filename: name of input file in asc format
+        :param str location: vertices or centroids, interpolation onto these locations
+        :param indices: None or a list of indices where interploation occurs
+        :param bool northern: Flag to specify northern or southern hemisphere
+        :param bool verbose: level of printed feedback 
         """
 
 
-        msg = "Function not implemented yet"
-        raise Exception(msg)
-
         msg = 'Filename must be a text string'
         assert isinstance(filename, str), msg
 
@@ -1502,7 +1516,7 @@ class Quantity(object):
         assert os.path.splitext(filename)[1] in ['.grd', '.asc'], msg
 
 
-        msg = "set_values_from_grd_file is only defined for location='vertices' or 'centroids'"
+        msg = "set_values_from_lat_long_grd_file is only defined for location='vertices' or 'centroids'"
         assert location in ['vertices', 'centroids'], msg
 
 
@@ -1525,7 +1539,7 @@ class Quantity(object):
         nrows = int(lines[1].split()[1].strip())
 
 
-        # Do cellsize (line 4) before line 2 and 3
+        # Parse cellsize (line 4) before line 2 and 3
         cellsize = float(lines[4].split()[1].strip())
 
         # Checks suggested by Joaquim Luis
@@ -1576,35 +1590,78 @@ class Quantity(object):
 
 
         if location == 'centroids':
-            points = self.domain.centroid_coordinates
+            if indices is None:
+                points = self.domain.centroid_coordinates
+            else:
+                indices = num.array(indices)
+                points = self.domain.centroid_coordinates[indices]
         else:
-            points = self.domain.vertex_coordinates
+            if indices is None:
+                points = self.domain.vertex_coordinates
+            else:
+                indices = num.array(indices)
+                points = self.domain.vertex_coordinates[tuple(indices),:]
 
-        from anuga.geospatial_data.geospatial_data import Geospatial_data,  ensure_absolute
-        points = ensure_absolute(points, geo_reference=self.domain.geo_reference)
+        from anuga.geospatial_data.geospatial_data import ensure_absolute
+        points = ensure_absolute(points, geo_reference=self.domain.geo_reference)               
 
-#         print numpy.max(points[:,0])
-#         print numpy.min(points[:,0])
-#         print numpy.max(points[:,1])
-#         print numpy.min(points[:,1])
-#
-#         print numpy.max(x)
-#         print numpy.min(x)
-#         print numpy.max(y)
-#         print numpy.min(y)
+        if verbose:
+            print (numpy.max(points[:,0]))
+            print (numpy.min(points[:,0]))
+            print (numpy.max(points[:,1]))
+            print (numpy.min(points[:,1]))
+            print (numpy.max(x))
+            print (numpy.min(x))
+            print (numpy.max(y))
+            print (numpy.min(y))
+
+            print (x.shape, x)
+            print (y.shape, y)
 
 
-        #print x.shape, x
-        #print y.shape, y
+        # We need to convert the UTM coordinates of the points to lat long
+        # then interpolate from the gridded data
+
+        if verbose:
+            print(self.domain.geo_reference)
+
+        utm_zone = self.domain.geo_reference.get_zone()
+
+        #import re
+        #utm_zone_number = re.findall(r'\d+', utm_zone)[0]
+        #utm_zone_letter = re.findall(r'[A-z]+', utm_zone)[0]
+
+        #print(utm_zone)
+        #print(points)
+
+        # we could use anuga's utmtoLL but it has not been vectorised so lets
+        # use this library, but we will have to download via pip
+        import utm
+        lat, long = utm.to_latlon(points[:,0], points[:,1], utm_zone, northern=northern)
+
+        #print(lat)
+        #print(long)
+
+        lat = num.reshape(lat, (-1,1))
+        long = num.reshape(long, (-1,1))
+        points_ll = num.hstack((long,lat))
 
 
+        # need to pull out the the utm zone number and letter
+
+        
+
+        #import utm
+        #points_ll = utm.to_latlon(easting = points[:,0], northing=points[:,1])
+
+
+        #print('points_ll', points_ll)
 
         from  anuga.fit_interpolate.interpolate2d import interpolate2d
-
         #print points
-        values = interpolate2d(x, y, Z, points, mode='linear', bounds_error=False)
+        values = interpolate2d(x, y, Z, points_ll, mode='linear', bounds_error=False)
 
-        #print values
+        #print ('values',values)
 
         # Call underlying method using array values
         if verbose:
@@ -1633,10 +1690,10 @@ class Quantity(object):
                 self.vertex_values[:] = values.reshape((-1,3))
             else:
                 msg = 'Number of values must match number of indices'
-                assert values.shape[0] == indices.shape[0], msg
+                assert values.shape[0] == len(indices), msg
 
                 # Brute force
-                self.vertex_values[indices] = values.reshape((-1,3))
+                self.vertex_values.reshape(-1,)[indices] = values
             # Cleanup centroid values
             self.interpolate()
 
